class A:
    ...


class A(object):
    ...


class A(
    object,
):
    ...


class A(
    object,
    #
):
    ...


class A(
    #
    object,
):
    ...


class A(
    #
    object
):
    ...


class A(
    object
    #
):
    ...


class A(
    #
    object,
    #
):
    ...


class A(
    #
    object,
    #
):
    ...


class A(
    #
    object
    #
):
    ...


class A(
    #
    object
    #
):
    ...


class B(A, object):
    ...


class B(object, A):
    ...


class B(
    object,
    A,
):
    ...


class B(
    A,
    object,
):
    ...


class B(
    object,
    # Comment on A.
    A,
):
    ...


class B(
    # Comment on A.
    A,
    object,
):
    ...


def f():
    class A(object):
        ...


class A(
    object,
):
    ...


class A(
    object,  # )
):
    ...


class A(
    object  # )
    ,
):
    ...


object = A


class B(object):
    ...



import typing


def f(x: typing.List[str]) -> None:
    ...


from typing import List


def f(x: List[str]) -> None:
    ...


import typing as t


def f(x: t.List[str]) -> None:
    ...


from typing import List as IList


def f(x: IList[str]) -> None:
    ...



for i in range(10):
    print(i)

print(i)  # name no longer defined on Python 3; no warning yet

for i in range(10):  # name not used within the loop; B007
    print(10)

print(i)  # name no longer defined on Python 3; no warning yet


for _ in range(10):  # _ is okay for a throw-away variable
    print(10)


for i in range(10):
    for j in range(10):
        for k in range(10):  # k not used, i and j used transitively
            print(i + j)


def strange_generator():
    for i in range(10):
        for j in range(10):
            for k in range(10):
                for l in range(10):
                    yield i, (j, (k, l))


for i, (j, (k, l)) in strange_generator():  # i, k not used
    print(j, l)



f"""
Should emit:
B021 - on lines 14, 22, 30, 38, 46, 54, 62, 70, 73
"""

VARIABLE = "world"


def foo1():
    """hello world!"""


def foo2():
    f"""hello {VARIABLE}!"""


class bar1:
    """hello world!"""


class bar2:
    f"""hello {VARIABLE}!"""


def foo1():
    """hello world!"""


def foo2():
    f"""hello {VARIABLE}!"""


class bar1:
    """hello world!"""


class bar2:
    f"""hello {VARIABLE}!"""


def foo1():
    "hello world!"


def foo2():
    f"hello {VARIABLE}!"


class bar1:
    "hello world!"


class bar2:
    f"hello {VARIABLE}!"


def foo1():
    "hello world!"


def foo2():
    f"hello {VARIABLE}!"


class bar1:
    "hello world!"


class bar2:
    f"hello {VARIABLE}!"


def baz():
    f"""I'm probably a docstring: {VARIABLE}!"""
    print(f"""I'm a normal string""")
    f"""Don't detect me!"""



"""
Should emit:
B009 - Line 18, 19, 20, 21, 22
B010 - Line 33, 34, 35, 36
"""

# Valid getattr usage
getattr(foo, bar)
getattr(foo, "bar", None)
getattr(foo, "bar{foo}".format(foo="a"), None)
getattr(foo, "bar{foo}".format(foo="a"))
getattr(foo, bar, None)
getattr(foo, "123abc")
getattr(foo, r"123\abc")
getattr(foo, "except")

# Invalid usage
getattr(foo, "bar")
getattr(foo, "_123abc")
getattr(foo, "abc123")
getattr(foo, r"abc123")
_ = lambda x: getattr(x, "bar")

# Valid setattr usage
setattr(foo, bar, None)
setattr(foo, "bar{foo}".format(foo="a"), None)
setattr(foo, "123abc", None)
setattr(foo, r"123\abc", None)
setattr(foo, "except", None)
_ = lambda x: setattr(x, "bar", 1)

# Invalid usage
setattr(foo, "bar", None)
setattr(foo, "_123abc", None)
setattr(foo, "abc123", None)
setattr(foo, r"abc123", None)



try:
    1 / 0
except ValueError as e:
    pass


try:
    1 / 0
except ValueError as e:
    print(e)


def f1():
    x = 1
    y = 2
    z = x + y


def f2():
    foo = (1, 2)
    (a, b) = (1, 2)

    bar = (1, 2)
    (c, d) = bar

    (x, y) = baz = bar


def f3():
    locals()
    x = 1


def f4():
    _ = 1
    __ = 1
    _discarded = 1



def a():
    try:
        pass
    finally:
        return  # warning


def b():
    try:
        pass
    finally:
        if 1 + 0 == 2 - 1:
            return  # warning


def c():
    try:
        pass
    finally:
        try:
            return  # warning
        except Exception:
            pass


def d():
    try:
        try:
            pass
        finally:
            return  # warning
    finally:
        pass


def e():
    if 1 == 2 - 1:
        try:

            def f():
                try:
                    pass
                finally:
                    return  # warning

        finally:
            pass


def g():
    try:
        pass
    finally:

        def h():
            return  # no warning

        e()


def i():
    while True:
        try:
            pass
        finally:
            break  # warning

            def j():
                while True:
                    break  # no warning


def h():
    while True:
        try:
            pass
        finally:
            continue  # warning

            def j():
                while True:
                    continue  # no warning


def k():
    try:
        pass
    finally:
        while True:
            break  # no warning
        while True:
            continue  # no warning
        while True:
            return  # warning


while True:
    try:
        pass
    finally:
        continue  # warning

while True:
    try:
        pass
    finally:
        break  # warning



"""A valid module docstring."""

from .expected import Expectation

expectation = Expectation()
expect = expectation.expect


_D213 = 'D213: Multi-line docstring summary should start at the second line'
_D400 = "D400: First line should end with a period (not '!')"


@expect(_D213)
@expect("D405: Section name should be properly capitalized "
        "('Returns', not 'returns')")
def not_capitalized():  # noqa: D416
    """Toggle the gizmo.

    returns
    -------
    A value of some sort.

    """


@expect(_D213)
@expect("D406: Section name should end with a newline "
        "('Returns', not 'Returns:')")
def superfluous_suffix():  # noqa: D416
    """Toggle the gizmo.

    Returns:
    -------
    A value of some sort.

    """


@expect(_D213)
@expect("D407: Missing dashed underline after section ('Returns')")
def no_underline():  # noqa: D416
    """Toggle the gizmo.

    Returns
    A value of some sort.

    """


@expect(_D213)
@expect("D407: Missing dashed underline after section ('Returns')")
@expect("D414: Section has no content ('Returns')")
def no_underline_and_no_description():  # noqa: D416
    """Toggle the gizmo.

    Returns

    """


@expect(_D213)
@expect("D410: Missing blank line after section ('Returns')")
@expect("D414: Section has no content ('Returns')")
@expect("D411: Missing blank line before section ('Yields')")
@expect("D414: Section has no content ('Yields')")
def consecutive_sections():  # noqa: D416
    """Toggle the gizmo.

    Returns
    -------
    Yields
    ------

    Raises
    ------
    Questions.

    """


@expect(_D213)
@expect("D408: Section underline should be in the line following the "
        "section's name ('Returns')")
def blank_line_before_underline():  # noqa: D416
    """Toggle the gizmo.

    Returns

    -------
    A value of some sort.

    """


@expect(_D213)
@expect("D409: Section underline should match the length of its name "
        "(Expected 7 dashes in section 'Returns', got 2)")
def bad_underline_length():  # noqa: D416
    """Toggle the gizmo.

    Returns
    --
    A value of some sort.

    """


@expect(_D213)
@expect("D413: Missing blank line after last section ('Returns')")
def no_blank_line_after_last_section():  # noqa: D416
    """Toggle the gizmo.

    Returns
    -------
    A value of some sort.
    """


@expect(_D213)
@expect("D411: Missing blank line before section ('Returns')")
def no_blank_line_before_section():  # noqa: D416
    """Toggle the gizmo.

    The function's description.
    Returns
    -------
    A value of some sort.

    """


@expect(_D213)
@expect("D214: Section is over-indented ('Returns')")
def section_overindented():  # noqa: D416
    """Toggle the gizmo.

        Returns
    -------
    A value of some sort.

    """


@expect(_D213)
@expect("D215: Section underline is over-indented (in section 'Returns')")
def section_underline_overindented():  # noqa: D416
    """Toggle the gizmo.

    Returns
        -------
    A value of some sort.

    """


@expect(_D213)
@expect("D215: Section underline is over-indented (in section 'Returns')")
@expect("D413: Missing blank line after last section ('Returns')")
@expect("D414: Section has no content ('Returns')")
def section_underline_overindented_and_contentless():  # noqa: D416
    """Toggle the gizmo.

    Returns
        -------
    """


@expect(_D213)
def ignore_non_actual_section():  # noqa: D416
    """Toggle the gizmo.

    This is the function's description, which will also specify what it
    returns

    """


@expect(_D213)
@expect("D401: First line should be in imperative mood "
        "(perhaps 'Return', not 'Returns')")
@expect("D400: First line should end with a period (not 's')")
@expect("D415: First line should end with a period, question "
        "mark, or exclamation point (not 's')")
@expect("D205: 1 blank line required between summary line and description "
        "(found 0)")
def section_name_in_first_line():  # noqa: D416
    """Returns
    -------
    A value of some sort.

    """


@expect(_D213)
@expect("D405: Section name should be properly capitalized "
        "('Short Summary', not 'Short summary')")
@expect("D412: No blank lines allowed between a section header and its "
        "content ('Short Summary')")
@expect("D409: Section underline should match the length of its name "
        "(Expected 7 dashes in section 'Returns', got 6)")
@expect("D410: Missing blank line after section ('Returns')")
@expect("D411: Missing blank line before section ('Raises')")
@expect("D406: Section name should end with a newline "
        "('Raises', not 'Raises:')")
@expect("D407: Missing dashed underline after section ('Raises')")
def multiple_sections():  # noqa: D416
    """Toggle the gizmo.

    Short summary
    -------------

    This is the function's description, which will also specify what it
    returns.

    Returns
    ------
    Many many wonderful things.
    Raises:
    My attention.

    """


@expect(_D213)
def false_positive_section_prefix():  # noqa: D416
    """Toggle the gizmo.

    Parameters
    ----------
    attributes_are_fun: attributes for the function.

    """


@expect(_D213)
def section_names_as_parameter_names():  # noqa: D416
    """Toggle the gizmo.

    Parameters
    ----------
    notes : list
        A list of wonderful notes.
    examples: list
        A list of horrible examples.

    """


@expect(_D213)
@expect("D414: Section has no content ('Returns')")
def valid_google_style_section():  # noqa: D406, D407
    """Toggle the gizmo.

    Args:
        note: A random string.

    Returns:

    Raises:
        RandomError: A random error that occurs randomly.

    """


@expect(_D213)
@expect("D416: Section name should end with a colon "
        "('Args:', not 'Args')")
def missing_colon_google_style_section():  # noqa: D406, D407
    """Toggle the gizmo.

    Args
        note: A random string.

    """


@expect("D417: Missing argument descriptions in the docstring "
        "(argument(s) y are missing descriptions in "
        "'bar' docstring)", func_name="bar")
def _test_nested_functions():
    x = 1

    def bar(y=2):  # noqa: D207, D213, D406, D407
        """Nested function test for docstrings.

        Will this work when referencing x?

        Args:
            x: Test something
that is broken.

        """
        print(x)


@expect(_D213)
@expect("D417: Missing argument descriptions in the docstring "
        "(argument(s) y are missing descriptions in "
        "'test_missing_google_args' docstring)")
def test_missing_google_args(x=1, y=2, _private=3):  # noqa: D406, D407
    """Toggle the gizmo.

    Args:
        x (int): The greatest integer.

    """


class TestGoogle:  # noqa: D203
    """Test class."""

    def test_method(self, test, another_test, _):  # noqa: D213, D407
        """Test a valid args section.

        Args:
            test: A parameter.
            another_test: Another parameter.

        """

    @expect("D417: Missing argument descriptions in the docstring "
            "(argument(s) test, y, z are missing descriptions in "
            "'test_missing_args' docstring)", arg_count=5)
    def test_missing_args(self, test, x, y, z=3, _private_arg=3):  # noqa: D213, D407
        """Test a valid args section.

        Args:
            x: Another parameter.

        """

    @classmethod
    @expect("D417: Missing argument descriptions in the docstring "
            "(argument(s) test, y, z are missing descriptions in "
            "'test_missing_args_class_method' docstring)", arg_count=5)
    def test_missing_args_class_method(cls, test, x, y, _, z=3):  # noqa: D213, D407
        """Test a valid args section.

        Args:
            x: Another parameter. The parameter below is missing description.
            y:

        """

    @staticmethod
    @expect("D417: Missing argument descriptions in the docstring "
            "(argument(s) a, y, z are missing descriptions in "
            "'test_missing_args_static_method' docstring)", arg_count=4)
    def test_missing_args_static_method(a, x, y, _test, z=3):  # noqa: D213, D407
        """Test a valid args section.

        Args:
            x: Another parameter.

        """

    @staticmethod
    @expect("D417: Missing argument descriptions in the docstring "
            "(argument(s) a, b are missing descriptions in "
            "'test_missing_docstring' docstring)", arg_count=2)
    def test_missing_docstring(a, b):  # noqa: D213, D407
        """Test a valid args section.

        Args:
            a:

        """

    @staticmethod
    def test_hanging_indent(skip, verbose):  # noqa: D213, D407
        """Do stuff.

        Args:
            skip (:attr:`.Skip`):
                Lorem ipsum dolor sit amet, consectetur adipiscing elit.
                Etiam at tellus a tellus faucibus maximus. Curabitur tellus
                mauris, semper id vehicula ac, feugiat ut tortor.
            verbose (bool):
                If True, print out as much infromation as possible.
                If False, print out concise "one-liner" information.

        """


@expect(_D213)
@expect("D417: Missing argument descriptions in the docstring "
        "(argument(s) y are missing descriptions in "
        "'test_missing_numpy_args' docstring)")
def test_missing_numpy_args(_private_arg=0, x=1, y=2):  # noqa: D406, D407
    """Toggle the gizmo.

    Parameters
    ----------
    x : int
        The greatest integer in the history \
of the entire world.

    """


class TestNumpy:  # noqa: D203
    """Test class."""

    def test_method(self, test, another_test, z, _, x=1, y=2, _private_arg=1):  # noqa: D213, D407
        """Test a valid args section.

        Some long string with a \
line continuation.

        Parameters
        ----------
        test, another_test
            Some parameters without type.
        z : some parameter with a very long type description that requires a \
line continuation.
            But no further description.
        x, y : int
            Some integer parameters.

        """

    @expect("D417: Missing argument descriptions in the docstring "
            "(argument(s) test, y, z are missing descriptions in "
            "'test_missing_args' docstring)", arg_count=5)
    def test_missing_args(self, test, x, y, z=3, t=1, _private=0):  # noqa: D213, D407
        """Test a valid args section.

        Parameters
        ----------
        x, t : int
            Some parameters.


        """

    @classmethod
    @expect("D417: Missing argument descriptions in the docstring "
            "(argument(s) test, y, z are missing descriptions in "
            "'test_missing_args_class_method' docstring)", arg_count=4)
    def test_missing_args_class_method(cls, test, x, y, z=3):  # noqa: D213, D407
        """Test a valid args section.

        Parameters
        ----------
        z
        x
            Another parameter. The parameters y, test below are
            missing descriptions. The parameter z above is also missing
            a description.
        y
        test

        """

    @staticmethod
    @expect("D417: Missing argument descriptions in the docstring "
            "(argument(s) a, z are missing descriptions in "
            "'test_missing_args_static_method' docstring)", arg_count=3)
    def test_missing_args_static_method(a, x, y, z=3, t=1):  # noqa: D213, D407
        """Test a valid args section.

        Parameters
        ----------
        x, y
            Another parameter.
        t : int
            Yet another parameter.

        """

    @staticmethod
    def test_mixing_numpy_and_google(danger):  # noqa: D213
        """Repro for #388.

        Parameters
        ----------
        danger
            Zoneeeeee!

        """


class TestIncorrectIndent:  # noqa: D203
    """Test class."""

    @expect("D417: Missing argument descriptions in the docstring "
            "(argument(s) y are missing descriptions in "
            "'test_incorrect_indent' docstring)", arg_count=3)
    def test_incorrect_indent(self, x=1, y=2):  # noqa: D207, D213, D407
        """Reproducing issue #437.

Testing this incorrectly indented docstring.

        Args:
            x: Test argument.

        """



"""
Should emit:
B003 - on line 9
"""

import os
from os import environ

os.environ = {}
environ = {}  # that's fine, assigning a new meaning to the module-level name


class Object:
    os = None


o = Object()
o.os.environ = {}



from typing import Any


# OK
def foo(a: int, *args: str, **kwargs: str) -> int:
    pass


# ANN401
def foo(a: Any, *args: str, **kwargs: str) -> int:
    pass


# ANN401
def foo(a: int, *args: str, **kwargs: str) -> Any:
    pass


# OK
def foo(a: int, *args: Any, **kwargs: Any) -> int:
    pass


# OK
def foo(a: int, *args: Any, **kwargs: str) -> int:
    pass


# ANN401
def foo(a: int, *args: str, **kwargs: Any) -> int:
    pass


class Bar:
    # OK
    def foo_method(self, a: int, *params: str, **options: str) -> int:
        pass

    # ANN401
    def foo_method(self, a: Any, *params: str, **options: str) -> int:
        pass

    # ANN401
    def foo_method(self, a: int, *params: str, **options: str) -> Any:
        pass

    # OK
    def foo_method(self, a: int, *params: Any, **options: Any) -> int:
        pass

    # OK
    def foo_method(self, a: int, *params: Any, **options: str) -> int:
        pass

    # OK
    def foo_method(self, a: int, *params: str, **options: Any) -> int:
        pass



#: E722
try:
    pass
except:
    pass
#: E722
try:
    pass
except Exception:
    pass
except:
    pass
#: E722
try:
    pass
except:
    pass
#: Okay
fake_code = """"
try:
    do_something()
except:
    pass
"""
try:
    pass
except Exception:
    pass
#: Okay
from . import compute_type

if compute_type(foo) == 5:
    pass



import collections
from collections import namedtuple


def f():
    lower = 0
    Camel = 0
    CONSTANT = 0
    _ = 0
    MyObj1 = collections.namedtuple("MyObj1", ["a", "b"])
    MyObj2 = namedtuple("MyObj12", ["a", "b"])



this_should_raise_Q003 = 'This is a \'string\''
this_is_fine = '"This" is a \'string\''
this_is_fine = "This is a 'string'"
this_is_fine = "\"This\" is a 'string'"
this_is_fine = r'This is a \'string\''



#: E714
if not X is Y:
    pass
#: E714
if not X.B is Y:
    pass
#: E714
if not X is Y is not Z:
    pass

#: Okay
if not X is not Y:
    pass

if x not in y:
    pass

if not (X in Y or X is Z):
    pass

if not (X in Y):
    pass

if x is not y:
    pass

if X is not Y is not Z:
    pass

if TrueElement.get_element(True) == TrueElement.get_element(False):
    pass

if (True) == TrueElement or x == TrueElement:
    pass

assert (not foo) in bar
assert {"x": not foo} in bar
assert [42, not foo] in bar



my_dict = {}
my_var = 0


def foo():
    my_var += 1


def bar():
    global my_var
    my_var += 1


def baz():
    global my_var
    global my_dict
    my_dict[my_var] += 1


def dec(x):
    return x


@dec
def f():
    dec = 1
    return dec



class bad:
    pass


class _bad:
    pass


class bad_class:
    pass


class Bad_Class:
    pass


class BAD_CLASS:
    pass


class Good:
    pass


class _Good:
    pass


class GoodClass:
    pass


class GOOD:
    pass



import collections
from collections import namedtuple

lower = 0
CONSTANT = 0
mixedCase = 0
_mixedCase = 0
mixed_Case = 0
myObj1 = collections.namedtuple("MyObj1", ["a", "b"])
myObj2 = namedtuple("MyObj2", ["a", "b"])



"""
Should emit:
B014 - on lines 11, 17, 28, 42, 49, 56, and 74.
"""

import binascii
import re

try:
    pass
except (Exception, TypeError):
    # TypeError is a subclass of Exception, so it doesn't add anything
    pass

try:
    pass
except (OSError, OSError) as err:
    # Duplicate exception types are useless
    pass


class MyError(Exception):
    pass


try:
    pass
except (MyError, MyError):
    # Detect duplicate non-builtin errors
    pass


try:
    pass
except (MyError, Exception) as e:
    # Don't assume that we're all subclasses of Exception
    pass


try:
    pass
except (MyError, BaseException) as e:
    # But we *can* assume that everything is a subclass of BaseException
    pass


try:
    pass
except (re.error, re.error):
    # Duplicate exception types as attributes
    pass


try:
    pass
except (IOError, EnvironmentError, OSError):
    # Detect if a primary exception and any its aliases are present.
    #
    # Since Python 3.3, IOError, EnvironmentError, WindowsError, mmap.error,
    # socket.error and select.error are aliases of OSError. See PEP 3151 for
    # more info.
    pass


try:
    pass
except (MyException, NotImplemented):
    # NotImplemented is not an exception, let's not crash on it.
    pass


try:
    pass
except (ValueError, binascii.Error):
    # binascii.Error is a subclass of ValueError.
    pass



from __future__ import annotations

from dataclasses import dataclass

from models import (
    Fruit,
    Nut,
)


@dataclass
class Foo:
    x: int
    y: int

    @classmethod
    def a(cls) -> Foo:
        return cls(x=0, y=0)

    @classmethod
    def b(cls) -> "Foo":
        return cls(x=0, y=0)

    @classmethod
    def c(cls) -> Bar:
        return cls(x=0, y=0)

    @classmethod
    def d(cls) -> Fruit:
        return cls(x=0, y=0)



"""
Should emit:
B002 - on lines 15 and 20
"""


def this_is_all_fine(n):
    x = n + 1
    y = 1 + n
    z = +x + y
    return +z


def this_is_buggy(n):
    x = ++n
    return x


def this_is_buggy_too(n):
    return ++n



from a import b
from a import BAD as DEF
from a import B
from a import Boo as DEF
from a import B as Abc
from a import B as A
from a import B as DEF
from a import b as a
from a import b as x
from a import b as c
from b import c
from a import b as d
from a import b as y
from b import C
from b import c as d

import A
import a
import b
import B

import x as y
import x as A
import x as Y
import x
import x as a



"""
Should emit:
B019 - on lines 73, 77, 81, 85, 89, 93, 97, 101
"""
import functools
from functools import cache, cached_property, lru_cache


def some_other_cache():
    ...


@functools.cache
def compute_func(self, y):
    ...


class Foo:
    def __init__(self, x):
        self.x = x

    def compute_method(self, y):
        ...

    @some_other_cache
    def user_cached_instance_method(self, y):
        ...

    @classmethod
    @functools.cache
    def cached_classmethod(cls, y):
        ...

    @classmethod
    @cache
    def other_cached_classmethod(cls, y):
        ...

    @classmethod
    @functools.lru_cache
    def lru_cached_classmethod(cls, y):
        ...

    @classmethod
    @lru_cache
    def other_lru_cached_classmethod(cls, y):
        ...

    @staticmethod
    @functools.cache
    def cached_staticmethod(y):
        ...

    @staticmethod
    @cache
    def other_cached_staticmethod(y):
        ...

    @staticmethod
    @functools.lru_cache
    def lru_cached_staticmethod(y):
        ...

    @staticmethod
    @lru_cache
    def other_lru_cached_staticmethod(y):
        ...

    @functools.cached_property
    def some_cached_property(self):
        ...

    @cached_property
    def some_other_cached_property(self):
        ...

    # Remaining methods should emit B019
    @functools.cache
    def cached_instance_method(self, y):
        ...

    @cache
    def another_cached_instance_method(self, y):
        ...

    @functools.cache()
    def called_cached_instance_method(self, y):
        ...

    @cache()
    def another_called_cached_instance_method(self, y):
        ...

    @functools.lru_cache
    def lru_cached_instance_method(self, y):
        ...

    @lru_cache
    def another_lru_cached_instance_method(self, y):
        ...

    @functools.lru_cache()
    def called_lru_cached_instance_method(self, y):
        ...

    @lru_cache()
    def another_called_lru_cached_instance_method(self, y):
        ...



"""Test case expected to be run with `mypy_init_return = True`."""

# Error
class Foo:
    def __init__(self):
        ...


# Error
class Foo:
    def __init__(self, foo):
        ...


# OK
class Foo:
    def __init__(self, foo) -> None:
        ...


# OK
class Foo:
    def __init__(self) -> None:
        ...


# OK
class Foo:
    def __init__(self, foo: int):
        ...


# OK
class Foo:
    def __init__(self, foo: int) -> None:
        ...


# Error
def __init__(self, foo: int):
    ...



import collections
import datetime as dt
import logging
import operator
import random
import re
import time
import types
from operator import attrgetter, itemgetter, methodcaller
from types import MappingProxyType


# B006
# Allow immutable literals/calls/comprehensions
def this_is_okay(value=(1, 2, 3)):
    ...


async def and_this_also(value=tuple()):
    pass


def frozenset_also_okay(value=frozenset()):
    pass


def mappingproxytype_okay(
    value=MappingProxyType({}), value2=types.MappingProxyType({})
):
    pass


def re_compile_ok(value=re.compile("foo")):
    pass


def operators_ok(
    v=operator.attrgetter("foo"),
    v2=operator.itemgetter("foo"),
    v3=operator.methodcaller("foo"),
):
    pass


def operators_ok_unqualified(
    v=attrgetter("foo"),
    v2=itemgetter("foo"),
    v3=methodcaller("foo"),
):
    pass


def kwonlyargs_immutable(*, value=()):
    ...


# Flag mutable literals/comprehensions


def this_is_wrong(value=[1, 2, 3]):
    ...


def this_is_also_wrong(value={}):
    ...


def and_this(value=set()):
    ...


def this_too(value=collections.OrderedDict()):
    ...


async def async_this_too(value=collections.defaultdict()):
    ...


def dont_forget_me(value=collections.deque()):
    ...


# N.B. we're also flagging the function call in the comprehension
def list_comprehension_also_not_okay(default=[i**2 for i in range(3)]):
    pass


def dict_comprehension_also_not_okay(default={i: i**2 for i in range(3)}):
    pass


def set_comprehension_also_not_okay(default={i**2 for i in range(3)}):
    pass


def kwonlyargs_mutable(*, value=[]):
    ...


# Recommended approach for mutable defaults
def do_this_instead(value=None):
    if value is None:
        value = set()


# B008
# Flag function calls as default args (including if they are part of a sub-expression)
def in_fact_all_calls_are_wrong(value=time.time()):
    ...


def f(when=dt.datetime.now() + dt.timedelta(days=7)):
    pass


def can_even_catch_lambdas(a=(lambda x: x)()):
    ...


# Recommended approach for function calls as default args
LOGGER = logging.getLogger(__name__)


def do_this_instead_of_calls_in_defaults(logger=LOGGER):
    # That makes it more obvious that this one value is reused.
    ...


# Handle inf/infinity/nan special case
def float_inf_okay(value=float("inf")):
    pass


def float_infinity_okay(value=float("infinity")):
    pass


def float_plus_infinity_okay(value=float("+infinity")):
    pass


def float_minus_inf_okay(value=float("-inf")):
    pass


def float_nan_okay(value=float("nan")):
    pass


def float_minus_NaN_okay(value=float("-NaN")):
    pass


def float_infinity_literal(value=float("1e999")):
    pass


# But don't allow standard floats
def float_int_is_wrong(value=float(3)):
    pass


def float_str_not_inf_or_nan_is_wrong(value=float("3.14")):
    pass


# B006 and B008
# We should handle arbitrary nesting of these B008.
def nested_combo(a=[float(3), dt.datetime.now()]):
    pass


# Don't flag nested B006 since we can't guarantee that
# it isn't made mutable by the outer operation.
def no_nested_b006(a=map(lambda s: s.upper(), ["a", "b", "c"])):
    pass


# B008-ception.
def nested_b008(a=random.randint(0, dt.datetime.now().year)):
    pass


# Ignore lambda contents since they are evaluated at call time.
def foo(f=lambda x: print(x)):
    f(1)



def get_name():
    return self.name


def get_name():
    return (self.name,)


def get_name():
    del self.name


def get_name(self):
    return self.name


x = list()


def randdec(maxprec, maxexp):
    return numeric_string(maxprec, maxexp)


def ternary_optarg(prec, exp_range, itr):
    for _ in range(100):
        a = randdec(prec, 2 * exp_range)
        b = randdec(prec, 2 * exp_range)
        c = randdec(prec, 2 * exp_range)
        yield a, b, c, None
        yield a, b, c, None, None


class Foo:
    CLASS_VAR = 1
    REFERENCES_CLASS_VAR = {"CLASS_VAR": CLASS_VAR}
    ANNOTATED_CLASS_VAR: int = 2


from typing import Literal


class Class:
    copy_on_model_validation: Literal["none", "deep", "shallow"]
    post_init_call: Literal["before_validation", "after_validation"]

    def __init__(self):
        Class


try:
    x = 1 / 0
except Exception as e:
    print(e)


y: int = 1

x: "Bar" = 1

[first] = ["yup"]


from typing import List, TypedDict


class Item(TypedDict):
    nodes: List[TypedDict("Node", {"name": str})]


from enum import Enum


class Ticket:
    class Status(Enum):
        OPEN = "OPEN"
        CLOSED = "CLOSED"

    def set_status(self, status: Status):
        self.status = status


def update_tomato():
    print(TOMATO)
    TOMATO = "cherry tomato"


A = f'{B}'
A = (
    f'B'
    f'{B}'
)


from typing import Annotated, Literal


def arbitrary_callable() -> None:
    ...


class PEP593Test:
    field: Annotated[
        int,
        "base64",
        arbitrary_callable(),
        123,
        (1, 2, 3),
    ]
    field_with_stringified_type: Annotated[
        "PEP593Test",
        123,
    ]
    field_with_undefined_stringified_type: Annotated[
        "PEP593Test123",
        123,
    ]
    field_with_nested_subscript: Annotated[
        dict[Literal["foo"], str],
        123,
    ]
    field_with_undefined_nested_subscript: Annotated[
        dict["foo", "bar"],  # Expected to fail as undefined.
        123,
    ]


def in_ipython_notebook() -> bool:
    try:
        # autoimported by notebooks
        get_ipython()  # type: ignore[name-defined]
    except NameError:
        return False  # not in notebook
    return True



import unittest


def Bad():
    pass


def _Bad():
    pass


def BAD():
    pass


def BAD_FUNC():
    pass


def good():
    pass


def _good():
    pass


def good_func():
    pass


def tearDownModule():
    pass


class Test(unittest.TestCase):
    def tearDown(self):
        return super().tearDown()

    def testTest(self):
        assert True



"""Test: import alias tracking."""
from typing import List

_ = List["Model"]


from typing import List as IList

_ = IList["Model"]


from collections.abc import ItemsView

_ = ItemsView["Model"]


import collections.abc

_ = collections.abc.ItemsView["Model"]


from collections import abc

_ = abc.ItemsView["Model"]



import collections
from collections import namedtuple


class C:
    lower = 0
    CONSTANT = 0
    mixedCase = 0
    _mixedCase = 0
    mixed_Case = 0
    myObj1 = collections.namedtuple("MyObj1", ["a", "b"])
    myObj2 = namedtuple("MyObj2", ["a", "b"])



try:
    pass
except:
    pass
except ValueError:
    pass

try:
    pass
except:
    pass
except ValueError:
    pass
finally:
    pass

try:
    pass
except:
    pass
except ValueError:
    pass
else:
    pass

try:
    pass
except:
    pass

try:
    pass
except ValueError:
    pass
except:
    pass

try:
    pass
except ValueError:
    pass

try:
    pass
finally:
    pass



def this_is_a_bug():
    o = object()
    if hasattr(o, "__call__"):
        print("Ooh, callable! Or is it?")
    if getattr(o, "__call__", False):
        print("Ooh, callable! Or is it?")


def this_is_fine():
    o = object()
    if callable(o):
        print("Ooh, this is actually callable.")



import StringIO
import glob
import os
import shutil
import tempfile
import time
from subprocess import PIPE, Popen, STDOUT
from module import Class, CONSTANT, function, BASIC, Apple
import foo
import FOO
import BAR
import bar



import sys
from sys import version, version as v

print(sys.version)

print(sys.version[:3])
print(version[:3])
print(v[:3])

# the tool is timid and only flags certain numeric slices
i = 3
print(sys.version[:i])



"""Test: typing_extensions module imports."""
from foo import Literal

# F821 Undefined name `Model`
x: Literal["Model"]

from typing_extensions import Literal


# OK
x: Literal["Model"]


import typing_extensions


# OK
x: typing_extensions.Literal["Model"]



"""Top-level docstring."""

__all__ = ["y"]
__version__: str = "0.1.0"

import a

try:
    import b
except ImportError:
    pass
else:
    pass

import c

if x > 0:
    import d
else:
    import e

y = x + 1

import f


def foo() -> None:
    import e


if __name__ == "__main__":
    import g



#!/usr/bin/env python
"""Wrapper around Flake8 to enable multiprocessing on all operating systems.

As of Python 3.8, macOS's default "start method" for multiprocessing is `spawn`. Flake8
requires a "start method" of `fork`, and disables multiprocessing if it detects `spawn`
or some other "start method". This script enables the `fork` start method before passing
along any command-line arguments to `flake8`.

This has never caused me any problems, but note that they disabled this for a reason:
Flake8's plugin interface doesn't work with `spawn`, and the maintainer says that `fork`
is "pretty broken" on macOS.

See:

- https://github.com/pycqa/flake8/issues/955
- https://github.com/PyCQA/flake8/issues/1337
- https://github.com/PyCQA/flake8/issues/342
- https://github.com/PyCQA/flake8/pull/1621

Example usage: python -m run_flake8 --select=E501 .
"""
import multiprocessing
import sys

from flake8.main import cli

if __name__ == "__main__":
    multiprocessing.set_start_method("fork", force=True)
    cli.main(sys.argv[1:])



"""Test case expected to be run with `suppress_dummy_args = True`."""

# OK
def foo(_) -> None:
    ...


# OK
def foo(*_) -> None:
    ...


# OK
def foo(**_) -> None:
    ...


# OK
def foo(a: int, _) -> None:
    ...


# OK
def foo() -> None:
    def bar(_) -> None:
        ...



from typing import List

import fastapi
from fastapi import Query


def okay(db=fastapi.Depends(get_db)):
    ...


def okay(data: List[str] = fastapi.Query(None)):
    ...


def okay(data: List[str] = Query(None)):
    ...


def error_due_to_missing_import(data: List[str] = Depends(None)):
    ...



assert 1 == 1

1 == 1

assert 1 in (1, 2)

1 in (1, 2)


if 1 == 2:
    pass


def test():
    assert 1 in (1, 2)

    1 in (1, 2)


data = [x for x in [1, 2, 3] if x in (1, 2)]


class TestClass:
    1 == 1


print(1 == 1)



import some as sum
from some import other as int

print = 1
copyright: 'annotation' = 2
(complex := 3)
float = object = 4
min, max = 5, 6

def bytes():
    pass

class slice:
    pass

try:
    ...
except ImportError as ValueError:
    ...

for memoryview, *bytearray in []:
    pass

with open('file') as str, open('file2') as (all, any):
    pass

[0 for sum in ()]



from typing import Any, Type

# Error
def foo(a, b):
    pass


# Error
def foo(a: int, b):
    pass


# Error
def foo(a: int, b) -> int:
    pass


# Error
def foo(a: int, b: int):
    pass


# Error
def foo():
    pass


# OK
def foo(a: int, b: int) -> int:
    pass


# OK
def foo() -> int:
    pass


# OK
def foo(a: int, *args: str, **kwargs: str) -> int:
    pass


# ANN401
def foo(a: Any, *args: str, **kwargs: str) -> int:
    pass


# ANN401
def foo(a: int, *args: str, **kwargs: str) -> Any:
    pass


# ANN401
def foo(a: int, *args: Any, **kwargs: Any) -> int:
    pass


# ANN401
def foo(a: int, *args: Any, **kwargs: str) -> int:
    pass


# ANN401
def foo(a: int, *args: str, **kwargs: Any) -> int:
    pass


class Foo:
    # OK
    def foo(self: "Foo", a: int, b: int) -> int:
        pass

    # ANN101
    def foo(self, a: int, b: int) -> int:
        pass

    # ANN401
    def foo(self: "Foo", a: Any, *params: str, **options: str) -> int:
        pass

    # ANN401
    def foo(self: "Foo", a: int, *params: str, **options: str) -> Any:
        pass

    # ANN401
    def foo(self: "Foo", a: int, *params: Any, **options: Any) -> int:
        pass

    # ANN401
    def foo(self: "Foo", a: int, *params: Any, **options: str) -> int:
        pass

    # ANN401
    def foo(self: "Foo", a: int, *params: str, **options: Any) -> int:
        pass

    # OK
    @classmethod
    def foo(cls: Type["Foo"], a: int, b: int) -> int:
        pass

    # ANN102
    @classmethod
    def foo(cls, a: int, b: int) -> int:
        pass



"""Test (edge case): accesses of object named `dict`."""

# OK
dict = {"parameters": {}}
dict["parameters"]["userId"] = "userId"
dict["parameters"]["itemId"] = "itemId"
del dict["parameters"]["itemId"]

# F821 Undefined name `key`
# F821 Undefined name `value`
x: dict["key", "value"]

# OK
x: dict[str, str]



"""Test case expected to be run with `suppress_none_returning = True`."""

# OK
def foo():
    a = 2 + 2


# OK
def foo():
    return


# OK
def foo():
    return None


# OK
def foo():
    a = 2 + 2
    if a == 4:
        return
    else:
        return


# OK
def foo():
    a = 2 + 2
    if a == 4:
        return None
    else:
        return


# OK
def foo():
    def bar() -> bool:
        return True

    bar()


# Error
def foo():
    return True


# Error
def foo():
    a = 2 + 2
    if a == 4:
        return True
    else:
        return
